@@ -70,20 +70,20 @@ class Foreperson:
             self.directory_path = directory_path
         else:
             raise FileNotFoundError("The experiment directory cannot be found.")
-        self.header = f' creator:\n   name:{self.creator['name']}\n'
-                      f'   affiliation:{self.creator['affiliation']}\n'
-                      f'   time:{self.creator['time']}\n data source:\n'
-                      f'   experiment:\n     '
-                      f'probe:{self.data_source['experiment']['probe']}\n     '
-                      f'instrument:{self.data_source['experiment']['instrument']}\n     '
-                      f'sample:{self.data_source['experiment']['sample']}\n '
-                      f'reduction:\n   software:{self.reduction['software']}\n   '
-                      f'yaml:{self.reduction['yaml']}\n data:\n   '
-                      f'column 1:{self.data['column 1']}\n   '
-                      f'column 2:{self.data['column 2']}\n   '
-                      f'column 3:{self.data['column 3']}'
+        self.header = ' creator:\n   name:'+self.creator['name']+'\n'
+                      '   affiliation:'+self.creator['affiliation']+'\n'
+                      '   time:'+self.creator['time']+'\n data source:\n'
+                      '   experiment:\n     '
+                      'probe:'+self.data_source['experiment']['probe']+'\n     '
+                      'instrument:'+self.data_source['experiment']['instrument']+'\n     '
+                      'sample:'+self.data_source['experiment']['sample']+'\n '
+                      'reduction:\n   software:'+self.reduction['software']+'\n   '
+                      'yaml:'+self.reduction['yaml']+'\n data:\n   '
+                      'column 1:'+self.data['column 1']+'\n   '
+                      'column 2:'+self.data['column 2']+'\n   '
+                      'column 3:'+self.data['column 3']+''
         if self.n_columns == 4:
-            self.header += f'\n   column 4:{self.data['column 4']}'
+            self.header += '\n   column 4:'+self.data['column 4']
 
 def reduce(run_numbers, yaml_file, directory='/dls/{}/data/{}/{}/', metadata=None):
     """
